@@ -1,15 +1,12 @@
-use std::{fmt::Debug, iter::once, str::FromStr};
-
-use smallvec::SmallVec;
-
 use crate::{
     definitions::{N_NODES_ON_STACK, N_VARS_ON_STACK},
-    operators::{BinOp, UnaryOp, VecOfUnaryFuncs},
+    expression::deep::{BinOpVec, BinOpsWithReprs, DeepEx, DeepNode, ExprIdxVec, UnaryOpWithReprs},
+    operators::{BinOp, Operator, UnaryOp, VecOfUnaryFuncs},
     parser::{ExParseError, Paren, ParsedToken},
-    Operator,
 };
+use std::{fmt::Debug, iter, str::FromStr};
 
-use super::deep::{BinOpVec, BinOpsWithReprs, DeepEx, DeepNode, ExprIdxVec, UnaryOpWithReprs};
+use smallvec::SmallVec;
 
 pub const ADD_REPR: &str = "+";
 pub const SUB_REPR: &str = "-";
@@ -57,10 +54,10 @@ pub fn find_overloaded_ops<'a, T: Copy>(all_ops: &[Operator<T>]) -> Option<Overl
 }
 
 pub fn parsed_tokens_to_deepex<'a, T: Copy + FromStr + Debug>(
-    parsed_tokens: &[ParsedToken<'a, T>]
+    parsed_tokens: &[ParsedToken<'a, T>],
 ) -> Result<DeepEx<'a, T>, ExParseError> {
     let mut found_vars = SmallVec::<[&str; N_VARS_ON_STACK]>::new();
-    let parsed_vars = parsed_tokens
+    let mut parsed_vars = parsed_tokens
         .iter()
         .filter_map(|pt| match pt {
             ParsedToken::Var(name) => {
@@ -74,7 +71,7 @@ pub fn parsed_tokens_to_deepex<'a, T: Copy + FromStr + Debug>(
             _ => None,
         })
         .collect::<SmallVec<[_; N_NODES_ON_STACK]>>();
-
+    parsed_vars.sort_unstable();
     let (expr, _) = make_expression(
         &parsed_tokens[0..],
         &parsed_vars,
@@ -129,7 +126,7 @@ where
     // variable 'tokens' from the outer scope
     let process_unary = |i: usize, uo, repr| {
         // gather subsequent unary operators from the beginning
-        let iter_of_uops = once((repr, uo)).chain(
+        let iter_of_uops = iter::once((repr, uo)).chain(
             (i + 1..parsed_tokens.len())
                 .map(|j| match parsed_tokens[j] {
                     ParsedToken::Op(op) => (op.repr, op.unary_op),
@@ -156,7 +153,7 @@ where
                 Paren::Open => {
                     let (expr, i_forward) = make_expression::<T>(
                         &parsed_tokens[i + n_uops + 1..],
-                        &parsed_vars,
+                        parsed_vars,
                         UnaryOpWithReprs {
                             reprs: vec_of_uop_reprs,
                             op: uop,
@@ -251,11 +248,11 @@ where
                     idx_tkn += 1;
                     let (expr, i_forward) = make_expression::<T>(
                         &parsed_tokens[idx_tkn..],
-                        &parsed_vars,
+                        parsed_vars,
                         UnaryOpWithReprs {
                             reprs: Vec::new(),
                             op: UnaryOp::new(),
-                        }
+                        },
                     )?;
                     nodes.push(DeepNode::Expr(expr));
                     idx_tkn += i_forward;
@@ -274,7 +271,7 @@ where
                 reprs: reprs_bin_ops,
                 ops: bin_ops,
             },
-            unary_ops
+            unary_ops,
         )?,
         idx_tkn,
     ))
