@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.9.0")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.9.1")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //! ```rust
@@ -139,7 +139,6 @@
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex;
-//!
 //! let expr = exmex::parse_with_default_ops::<f64>("x^2 + y^2")?;
 //! let dexpr_dx = expr.clone().partial(0)?;
 //! let dexpr_dy = expr.partial(1)?;
@@ -170,15 +169,15 @@
 //!
 //! An instance of [`FlatEx`](FlatEx) can be displayed as string. Note that this
 //! [`unparse`](FlatEx::unparse)d string does not necessarily coincide with the original
-//! string, since curly brackets are added.
+//! string, since, e.g., curly brackets are added and expressions are compiled.
 //!
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex;
-//! let flatex = exmex::parse_with_default_ops::<f64>("-sin(z)/cos(mother_of_names)")?;
-//! assert_eq!(format!("{}", flatex), "-(sin({z}))/cos({mother_of_names})");
+//! let flatex = exmex::parse_with_default_ops::<f64>("-sin(z)/cos(mother_of_names) + 2^7")?;
+//! assert_eq!(format!("{}", flatex), "-(sin({z}))/cos({mother_of_names})+128.0");
 //! #
 //! #     Ok(())
 //! # }
@@ -304,18 +303,37 @@ mod tests {
         util::{assert_float_eq_f32, assert_float_eq_f64},
         ExParseError, FlatEx,
     };
-
+    
     #[test]
     fn test_readme() {
-        fn readme() -> Result<f64, ExParseError> {
+        fn readme_partial () -> Result<(), ExParseError> {
+            let expr = parse_with_default_ops::<f64>("y*x^2")?;
+            
+            // d_x
+            let dexpr_dx = expr.partial(0)?;
+            assert_eq!(format!("{}", dexpr_dx), "({x}*2.0)*{y}");
+            
+            // d_xy
+            let ddexpr_dxy = dexpr_dx.partial(1)?;
+            assert_eq!(format!("{}", ddexpr_dxy), "{x}*2.0");
+            assert_float_eq_f64(ddexpr_dxy.eval(&[2.0, f64::MAX])?, 4.0);
+            
+            // d_xyx
+            let dddexpr_dxyx = ddexpr_dxy.partial(0)?;
+            assert_eq!(format!("{}", dddexpr_dxyx), "2.0");
+            assert_float_eq_f64(dddexpr_dxyx.eval(&[f64::MAX, f64::MAX])?, 2.0);
+            
+            Ok(())
+        }
+        fn readme() -> Result<(), ExParseError> {
             let result = eval_str("sin(73)")?;
             assert_float_eq_f64(result, 73f64.sin());
             let expr = parse_with_default_ops::<f64>("2*x^3-4/z")?;
             let value = expr.eval(&[5.3, 0.5])?;
             assert_float_eq_f64(value, 289.75399999999996);
-            Ok(value)
+            Ok(())
         }
-        fn readme_int() -> Result<u32, ExParseError> {
+        fn readme_int() -> Result<(), ExParseError> {
             let ops = vec![
                 Operator {
                     repr: "|",
@@ -334,19 +352,28 @@ mod tests {
             let expr = parse::<u32>("!(a|b)", &ops)?;
             let result = expr.eval(&[0, 1])?;
             assert_eq!(result, u32::MAX - 1);
-            Ok(result)
+            Ok(())
         }
+        assert!(!readme_partial().is_err());
         assert!(!readme().is_err());
         assert!(!readme_int().is_err());
     }
     #[test]
-    fn test_variables_curly() {
-        let sut = "5*{x} + 4*log2(log(1.5-{gamma}))*({x}*-(tan(cos(sin(652.2-{gamma}))))) + 3*{x}";
+    fn test_variables_curly_space_names() {
+        let sut = "{x } + { y }";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
-        assert_float_eq_f64(expr.eval(&[1.0, 0.0]).unwrap(), 11.429314405093656);
-        let sut = "2*(4*{x} + y^2)";
+        assert_float_eq_f64(expr.eval(&[1.0, 1.0]).unwrap(), 2.0);
+        assert_eq!(expr.unparse().unwrap(), "{x }+{ y }");
+        let sut = "2*(4*{ xasd sa } + { y z}^2)";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
         assert_float_eq_f64(expr.eval(&[2.0, 3.0]).unwrap(), 34.0);
+        assert_eq!(expr.unparse().unwrap(), "2.0*(4.0*{ xasd sa }+{ y z}^2.0)");
+    }
+    #[test]
+    fn test_variables_curly() {
+        let sut = "5*{x} +  4*log2(log(1.5+{gamma}))*({x}*-(tan(cos(sin(652.2-{gamma}))))) + 3*{x}";
+        let expr = parse_with_default_ops::<f64>(sut).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.2, 1.0]).unwrap(), 8.040556934857268);
 
         let sut = "sin({myvwmlf4i58eo;w/-sin(a)r_25})";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
@@ -358,7 +385,7 @@ mod tests {
     }
     #[test]
     fn test_variables() {
-        let sut = "sin({x})+(((cos({y})^(sin({z})))*log(cos({y})))*cos({z}))";
+        let sut = "sin  ({x})+(((cos({y})   ^  (sin({z})))*log(cos({y})))*cos({z}))";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
         let reference =
             |x: f64, y: f64, z: f64| x.sin() + y.cos().powf(z.sin()) * y.cos().ln() * z.cos();
@@ -378,7 +405,7 @@ mod tests {
             reference(1.0, 2.0, 4.0),
         );
 
-        let sut = "0.02*sin(-(3*(2*(5.0 - 1/z))))";
+        let sut = "0.02*sin( - (3*(2*(5.0 - 1/z))))";
         let expr = parse_with_default_ops::<f64>(sut).unwrap();
         let reference = |z: f64| 0.02 * (-(3.0 * (2.0 * (5.0 - 1.0 / z)))).sin();
         assert_float_eq_f64(expr.eval(&[4.0]).unwrap(), reference(4.0));
@@ -536,12 +563,90 @@ mod tests {
         ) {
             println!("partial {}", deri);
             let mut rng = rand::thread_rng();
-            let vut = rng.gen_range(random_range);
-            let mut vars: SmallVec<[f64; 10]> = smallvec![0.0; n_vars];
-            vars[var_idx] = vut;
-            println!("value under test {}.", vut);
-            assert_float_eq_f64(deri.eval(&vars).unwrap(), reference(vut));
+            for _ in 0..3 {
+                let vut = rng.gen_range(random_range.clone());
+                let mut vars: SmallVec<[f64; 10]> = smallvec![0.0; n_vars];
+                vars[var_idx] = vut;
+                println!("value under test {}.", vut);
+                assert_float_eq_f64(deri.eval(&vars).unwrap(), reference(vut));
+            }
         }
+        
+        let sut = "+x";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |_: f64| 1.0;
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+        let sut = "++x";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |_: f64| 1.0;
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+        let sut = "+-+x";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |_: f64| -1.0;
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+        let sut = "-x";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |_: f64| -1.0;
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+
+        let sut = "--x";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |_: f64| 1.0;
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+        let sut = "sin(sin(x))";
+        println!("{}", sut);
+        let var_idx = 0;
+        let n_vars = 1;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |x: f64| x.sin().cos() * x.cos();
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+
+        let sut = "sin(x)-cos(x)+a";
+        println!("{}", sut);
+        let var_idx = 1;
+        let n_vars = 2;
+        let flatex_1 = parse_with_default_ops::<f64>(sut).unwrap();
+        let deri = flatex_1.partial(var_idx).unwrap();
+        let reference = |x: f64| x.cos() + x.sin();
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        println!("{}", deri);
+        let deri = deri.partial(var_idx).unwrap();
+        let reference = |x: f64| -x.sin() + x.cos();
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        println!("{}", deri);
+        let deri = deri.partial(var_idx).unwrap();
+        let reference = |x: f64| -x.cos() - x.sin();
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
+        println!("{}", deri);
+        let deri = deri.partial(var_idx).unwrap();
+        let reference = |x: f64| x.sin() - x.cos();
+        test(var_idx, n_vars, -10000.0..10000.0, &deri, reference);
 
         let sut = "sin(x)-cos(x)+tan(x)+a";
         println!("{}", sut);
